/*
 * Copyright (c) 2019 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <stdint.h>

static const uint32_t FTFA_FSTAT = 0x40020000;

static const uint8_t FTFA_FSTAT_CCIF = 1 << 7;

__attribute__((section(".data")))
static uint8_t
run_command(void)
{
    volatile uint8_t *fstat = (uint8_t *) FTFA_FSTAT;

    /* Start command. */
    *fstat = FTFA_FSTAT_CCIF;

    /* Wait for it to finish. */
    for (;;) {
        uint8_t v = *fstat;

        if ((v & FTFA_FSTAT_CCIF) != 0) {
            return v;
        }
    }
}

uint8_t
flash_run_command(void)
{
    asm volatile ("cpsid i");

    uint8_t status = run_command();

    asm volatile ("cpsie i");

    return status;
}
