/*
 * Copyright (c) 2019 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

use nvic;
use clock;

extern {
    fn USB_VcomInit(vid: u16, pid: u16);
    fn USB_VcomDeinit();
}

pub unsafe fn init(vid: u16, pid: u16) {
    USB_VcomInit(vid, pid);
}

pub unsafe fn shutdown() {
    USB_VcomDeinit();
}

#[no_mangle]
#[allow(non_snake_case)]
pub unsafe extern fn USB_DeviceIsrEnable()
{
    let irq_number = 24; // USB0
    let irq_prio = 3; // Stolen from NXP example.

    nvic::set_priority(irq_number, irq_prio);
    nvic::enable_irq(irq_number);
}

#[no_mangle]
#[allow(non_snake_case)]
pub unsafe extern fn USB_DeviceIsrDisable()
{
    let irq_number = 24; // USB0

    nvic::disable_irq(irq_number);
}

#[no_mangle]
#[allow(non_snake_case)]
pub unsafe extern fn USB_DeviceClockInit()
{
    clock::configure_usb();
}
